package com.googlecode.marrowboy.assertionresult;

import java.util.ArrayList;
import java.util.List;

/**
 * Encapsulates a single test method, (possibly) containing multiple test
 * assertions.
 * 
 * @author mjg
 */
public class MethodResult {

	private final String name;
	private final List<AssertionResult> assertionResults;

	/**
	 * Constructor
	 * 
	 * @param methodName
	 *            the name of the method
	 */
	public MethodResult(String methodName) {
		assertionResults = new ArrayList<AssertionResult>();
		this.name = methodName;
	}

	/**
	 * Getter
	 * 
	 * @return the name of the method
	 */
	public String getName() {
		return name;
	}

	/**
	 * Add an {@link AssertionResult} to this method
	 * 
	 * @param assertionResult
	 *            the {@link AssertionResult} to add
	 */
	public void addAssertion(AssertionResult assertionResult) {
		assertionResults.add(assertionResult);
	}

	/**
	 * How many assertions passed in this method?
	 * 
	 * @return the number of assertions which passed
	 */
	public int getPassCount() {
		int count = 0;
		for (AssertionResult assertionResult : assertionResults) {
			if (assertionResult.wasSuccessful()) {
				count++;
			}
		}
		return count;
	}

	/**
	 * How many assertions failed in this method?
	 * 
	 * @return the number of assertions which failed
	 */
	public int getFailCount() {
		int count = 0;
		for (AssertionResult assertionResult : assertionResults) {
			if (!assertionResult.wasSuccessful()) {
				count++;
			}
		}
		return count;
	}

	/**
	 * How many assertions in this class?
	 * 
	 * @return the number of assertions
	 */
	public int getTotalCount() {
		return assertionResults.size();
	}

	/**
	 * Convenience method to check if all assertions passed
	 * 
	 * @return whether all the assertions passed
	 */
	public boolean allPassed() {
		return (getFailCount() == 0);
	}

	/**
	 * Getter
	 * 
	 * @return A {@link List} of {@link AssertionResult}s in this method
	 */
	public List<AssertionResult> getAssertionResults() {
		return assertionResults;
	}

}
