package com.googlecode.marrowboy.writers;

import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.PrintStream;
import java.io.Writer;
import java.util.Date;

import org.apache.commons.lang.StringEscapeUtils;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.VelocityEngine;

import com.googlecode.marrowboy.assertionresult.TestRunResult;

/**
 * Implementation of {@link ReportWriter} which uses Apache Velocity template
 * library.
 * 
 * @author mjg
 */
public class VelocityTemplateWriter implements ReportWriter {

	public static final String DEFAULT_HTML_TEMPLATE = "defaultHtmlTemplate.vm";
	public static final String DEFAULT_XML_TEMPLATE = "defaultXmlTemplate.vm";
	public static final String DEFAULT_PLAIN_TEXT_TEMPLATE = "defaultTxtTemplate.vm";

	private final CssProvider cssProvider;
	private final String templateName;

	/**
	 * Constructor.
	 */
	public VelocityTemplateWriter(String templateName) {
		cssProvider = new DefaultCssProvider();
		this.templateName = templateName;
	}

	/**
	 * {@inheritDoc}
	 * 
	 * Any exceptions thrown by Velocity are wrapped with
	 * {@link RuntimeException}s
	 */
	public void writeReport(TestRunResult testResults, PrintStream stream) {

		VelocityEngine ve = new VelocityEngine();
		ve.setProperty("resource.loader", "class");
		ve.setProperty("class.resource.loader.class", "org.apache.velocity.runtime.resource.loader.ClasspathResourceLoader");

		try {
			ve.init();

		} catch (Exception e) {
			throw new RuntimeException("Exception caught while initialising Velocity", e);

		}

		VelocityContext context = new VelocityContext();
		context.put("css", cssProvider.getCss());
		context.put("dateTime", new Date().toString());
		context.put("results", testResults);
		context.put("util", new StringEscapeUtils());
		context.put("newline", System.getProperty("line.separator"));

		Writer outputWriter = new OutputStreamWriter(stream);

		try {
			ve.mergeTemplate(templateName, "utf-8", context, outputWriter);

		} catch (Exception e) {
			throw new RuntimeException("Exception caught while merging Velocity", e);

		}

		try {
			outputWriter.flush();
			outputWriter.close();

		} catch (IOException e) {
			throw new RuntimeException("Exception caught while closing output stream", e);

		}

	}
}
